use std::collections::BTreeSet;
use std::fmt;
use std::iter::FromIterator;

use super::Scalar;

// an unknown variable with an id
#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
pub struct Unknown(pub i64);

pub type UnknownSet = BTreeSet<Unknown>;

pub trait Unknowns {
    fn unknowns(&self) -> UnknownSet;
    fn has_unknowns(&self) -> bool;
    fn has_unknown(&self, u: Unknown) -> bool;
}

impl Unknowns for Scalar {
    fn unknowns(&self) -> UnknownSet {
        UnknownSet::new()
    }
    fn has_unknowns(&self) -> bool {
        false
    }
    fn has_unknown(&self, _: Unknown) -> bool {
        false
    }
}

impl Unknowns for Unknown {
    fn unknowns(&self) -> UnknownSet {
        FromIterator::from_iter(Some(*self))
    }
    fn has_unknowns(&self) -> bool {
        true
    }
    fn has_unknown(&self, u: Unknown) -> bool {
        *self == u
    }
}

impl fmt::Display for Unknown {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "u{}", self.0)
    }
}
